package access_log

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/common.in/utils"
	"gd_access_log/rpc_apis"
	"gd_access_log/rpc_apis/gd_management"
	"encoding/json"
	"fmt"
	"github.com/astaxie/beego/orm"
	"go.uber.org/zap"
	"time"
)

func writeInvalidAccessLog(ctx context.Context, req *apis.LogAddInvalidAccessLogReq) error {
	var accessLog apis.InvalidAccessLog
	var rpcCount int
	var dbCount int
	for {
		mReq := gd_management.ManagementGetNameReq{}
		mReq.MerchantId = req.MerchantId
		mReq.ApiId = req.ApiId
		mreply, err := rpc_apis.Management.ManagementGetName(ctx, &mReq)
		if err != nil {
			rpcCount++
			if rpcCount > MaxTimes {
				l.Error("func",
					zap.String("call", "ManagementGetName"),
					zap.String("args", utils.MarshalJsonString(mReq)),
					zap.String("error", err.Error()))
				return err
			}
			time.Sleep(time.Second)
			continue
		}

		accessLog.Code = req.Code
		accessLog.Msg = req.Msg
		accessLog.ApiId = req.ApiId
		accessLog.RemoteAddr = req.RemoteAddr
		accessLog.MerchantId = req.MerchantId
		accessLog.Elapsed = Decimal(req.Elapsed)
		accessLog.RequestParams = req.RequestParams
		accessLog.Timestamp = req.TimeStamp
		accessLog.ApiName = mreply.ApiName
		accessLog.MerchantName = mreply.MerchantName

		timeNow := time.Now()
		var timeLayout = "2006-01-02"

		accessLog.CreateTime = timeNow.Format(timeLayout)
		for {
			_, err = orm.NewOrm().Insert(&accessLog)
			if err != nil {
				dbCount++
				if dbCount > MaxTimes {
					l.Error("mysql",
						zap.String("sql", "insert  t_gd_access_log_invalid"),
						zap.String("args", utils.MarshalJsonString(accessLog)),
						zap.String("error", err.Error()))
					return err
				}
				time.Sleep(time.Second)
				continue
			}
			break
		}
		break
	}
	return nil
}

func LogAddInvalidAccessLog(ctx context.Context, req *apis.LogAddInvalidAccessLogReq, reply *apis.LogAddInvalidAccessLogReply) error {
	d, _ := json.Marshal(*req)
	fmt.Println(string(d))
	err := writeInvalidAccessLog(ctx, req)
	if err != nil {
		l.Error("access log",
			zap.String("args", utils.MarshalJsonString(*req)),
			zap.String("err", err.Error()))
	}
	return nil
}
