package h5_access_log

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/common.in/utils"
	"gd_access_log/errors"
	"gd_access_log/rpc_apis"
	"gd_access_log/rpc_apis/gd_management"
	"encoding/json"
	"fmt"
	"github.com/astaxie/beego/orm"
	"go.uber.org/zap"
	"time"
)

var MaxTimes = 5

func writeAccessLog(ctx context.Context, req *apis.H5LogAddAccessLogReq) (int64, apis.H5AccessLog, error) {
	var accessLog apis.H5AccessLog
	var accessId int64
	var rpcCount int
	var dbCount int
	for {
		mReq := gd_management.ManagementGetNameReq{}
		mReq.MerchantId = req.MerchantId
		mReq.ApiId = req.ApiId
		mReq.H5ApiId = req.H5ApiId
		mReq.H5ServiceId = req.H5ServiceId

		mreply, err := rpc_apis.Management.ManagementGetName(ctx, &mReq)
		if err != nil {
			rpcCount++
			if rpcCount > MaxTimes {
				l.Error("func",
					zap.String("call", "ManagementGetName"),
					zap.String("args", utils.MarshalJsonString(mReq)),
					zap.String("error", err.Error()))
				return 0, accessLog, err
			}
			time.Sleep(time.Second)
			continue
		}

		accessLog.Code = req.AccessLogWrite.Code
		accessLog.Msg = req.AccessLogWrite.Msg
		accessLog.State = req.AccessLogWrite.State
		accessLog.ApiId = req.ApiId
		accessLog.MerchantId = req.MerchantId
		accessLog.H5ApiId = req.H5ApiId
		accessLog.H5ServiceId = req.H5ServiceId
		accessLog.PlateNo = req.PlateNo
		accessLog.PlateType = req.PlateType
		accessLog.RemoteAddr = req.RemoteAddr
		accessLog.Vin = req.Vin
		accessLog.Elapsed = Decimal(req.AccessLogWrite.Elapsed)
		accessLog.IsReuse = req.AccessLogWrite.IsReuse
		accessLog.RequestParams = req.AccessLogWrite.RequestParams
		if len(req.AccessLogWrite.ResponseParams) < 4096 {
			accessLog.ResponseParams = req.AccessLogWrite.ResponseParams
		}
		accessLog.RawRequestParams = req.AccessLogWrite.RawRequestParams
		if len(req.AccessLogWrite.RawResponseParams) < 4096 {
			accessLog.RawResponseParams = req.AccessLogWrite.RawResponseParams
		}
		accessLog.Timestamp = req.TimeStamp
		accessLog.ApiName = mreply.ApiName
		accessLog.MerchantName = mreply.MerchantName
		accessLog.H5ApiName = mreply.H5ApiName
		accessLog.H5ServiceName = mreply.H5ServiceName
		timeNow := time.Now()
		var timeLayout = "2006-01-02"

		accessLog.CreateTime = timeNow.Format(timeLayout)
		for {
			accessId, err = orm.NewOrm().Insert(&accessLog)
			if err != nil {
				dbCount++
				if dbCount > MaxTimes {
					l.Error("mysql",
						zap.String("sql", "insert access log"),
						zap.String("args", utils.MarshalJsonString(accessLog)),
						zap.String("error", err.Error()))
					return 0, accessLog, err
				}
				time.Sleep(time.Second)
				continue
			}
			break
		}
		break
	}
	return accessId, accessLog, nil
}

func writeThirdpartAccessLog(ctx context.Context, req *apis.H5LogAddAccessLogReq, id int64, log apis.H5AccessLog) error {
	var rpcCount int
	var dbCount int
	for index, _ := range req.H5ThirdpartLogWrites {
		for {
			var accessLog apis.H5ThirdpartLog
			mReq := gd_management.ManagementGetNameReq{}
			mReq.ProviderApiId = req.H5ThirdpartLogWrites[index].ProviderApiId
			mreply, err := rpc_apis.Management.ManagementGetName(ctx, &mReq)
			if err != nil {
				rpcCount++
				if rpcCount > MaxTimes {
					//TODO
					l.Error("func",
						zap.String("call", "ManagementGetName"),
						zap.String("args", utils.MarshalJsonString(mReq)),
						zap.String("error", err.Error()))
					return err
				}
				time.Sleep(time.Second)
				continue
			}

			accessLog.State = req.H5ThirdpartLogWrites[index].State
			accessLog.ProviderId = mreply.ProviderId
			accessLog.ProviderApiId = req.H5ThirdpartLogWrites[index].ProviderApiId
			accessLog.ApiId = req.ApiId
			accessLog.PlateNo = req.PlateNo
			accessLog.PlateType = req.PlateType
			accessLog.RemoteAddr = req.RemoteAddr
			accessLog.Vin = req.Vin
			accessLog.Code = req.H5ThirdpartLogWrites[index].Code
			accessLog.Elapsed = Decimal(req.H5ThirdpartLogWrites[index].Elapsed)
			accessLog.MerchantId = req.MerchantId
			accessLog.Msg = req.H5ThirdpartLogWrites[index].Msg
			accessLog.RequestParams = req.H5ThirdpartLogWrites[index].RequestParams
			if len(req.H5ThirdpartLogWrites[index].ResponseParams) < 4096 {

				accessLog.ResponseParams = req.H5ThirdpartLogWrites[index].ResponseParams
			}
			accessLog.ApiName = log.ApiName
			accessLog.MerchantName = log.MerchantName
			accessLog.H5ServiceName = log.H5ServiceName
			accessLog.H5ApiName = log.H5ApiName
			accessLog.H5ServiceId = log.H5ServiceId
			accessLog.H5ApiId = log.H5ApiId
			accessLog.ProviderApiName = mreply.ProviderApiName
			accessLog.ProviderName = mreply.ProviderName
			accessLog.Timestamp = req.TimeStamp
			accessLog.RemoteAddr = req.RemoteAddr
			accessLog.AccessId = id
			timeNow := time.Now()
			var timeLayout = "2006-01-02"
			accessLog.CreateTime = timeNow.Format(timeLayout)
			for {
				_, err = orm.NewOrm().Insert(&accessLog)
				if err != nil {
					dbCount++
					if dbCount > MaxTimes {
						l.Error("mysql",
							zap.String("sql", "insert t_gd_thirdpart_access_log"),
							zap.String("args", utils.MarshalJsonString(accessLog)),
							zap.String("error", err.Error()))
						return errors.DataBaseError
					}
					time.Sleep(time.Second)
					continue
				}
				break
			}
			break
		}
	}
	return nil
}

func H5LogAddAccessLog(ctx context.Context, req *apis.H5LogAddAccessLogReq, reply *apis.H5LogAddAccessLogReply) error {
	d, _ := json.Marshal(*req)
	fmt.Println(string(d))
	id, accessLog, err := writeAccessLog(ctx, req)
	if err != nil {
		l.Error("h5 access log",
			zap.String("args", utils.MarshalJsonString(*req)),
			zap.String("err", err.Error()))
	}

	err = writeThirdpartAccessLog(ctx, req, id, accessLog)
	if err != nil {
		l.Error("h5 thirdparty access log",
			zap.String("args", utils.MarshalJsonString(*req)),
			zap.String("err", err.Error()))
	}
	return nil
}
