package h5_access_log

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/errors"
	"fmt"
	"github.com/astaxie/beego/orm"
	"time"
)

func getAccesslogWhere(req *apis.H5LogQueryAccessLogReq, querySet orm.QuerySeter) orm.QuerySeter {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		cond := orm.NewCondition()
		cond = cond.And("timestamp__gte", req.StartTimestamp).And("timestamp__lte", req.EndTimestamp)
		querySet = querySet.SetCond(cond)
	}
	if req.ApiId != 0 {
		querySet = querySet.Filter("api_id", req.ApiId)
	}
	if req.PlateNo != "" {
		querySet = querySet.Filter("plate_no", req.PlateNo)
	}
	if req.Vin != "" {
		querySet = querySet.Filter("vin", req.Vin)
	}
	if req.MerchantId != 0 {
		querySet = querySet.Filter("merchant_id", req.MerchantId)
	}

	if req.H5ApiId != 0 {
		querySet = querySet.Filter("h5_api_id", req.H5ApiId)
	}

	if req.H5ServiceId != 0 {
		querySet = querySet.Filter("h5_service_id", req.H5ServiceId)
	}

	return querySet
}

func H5LogQueryAccessLog(ctx context.Context, req *apis.H5LogQueryAccessLogReq, reply *apis.H5LogQueryAccessLogReply) error {

	querySet := orm.NewOrm().QueryTable("t_gd_h5_access_log")
	querySet = getAccesslogWhere(req, querySet)

	count, err := querySet.Count()
	if err != nil {
		return errors.DataBaseError
	}

	fmt.Println(count)

	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	reply.Total = count
	_, err = querySet.OrderBy("-timestamp").Limit(req.PageSize, (req.PageNumber-1)*req.PageSize).All(&reply.H5AccessLogs)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}
	for i, _ := range reply.H5AccessLogs {
		var timeLayout = "2006-01-02 15:04:05"
		reply.H5AccessLogs[i].CreateTime = time.Unix(reply.H5AccessLogs[i].Timestamp, 0).Format(timeLayout)
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	fmt.Println(reply)
	return nil
}
