package h5_access_log

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/errors"
	"fmt"
	"github.com/astaxie/beego/orm"
	"strconv"
)

func getAccesslogCountWhere(req *apis.H5LogQueryAccessLogCountReq) (w string, s []interface{}) {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		w += " and " + "timestamp between" + " ? and ?"
		s = append(s, req.StartTimestamp, req.EndTimestamp)
	}
	if req.ApiId != 0 {
		w += " and " + "api_id" + " = ?"
		s = append(s, req.ApiId)
	}
	if req.MerchantId != 0 {
		w += " and " + "merchant_id" + " = ?"
		s = append(s, req.MerchantId)
	}

	if req.H5ServiceId != 0 {
		w += " and " + "h5_service_id" + " = ?"
		s = append(s, req.H5ServiceId)
	}

	if req.H5ApiId != 0 {
		w += " and " + "h5_api_id" + " = ?"
		s = append(s, req.H5ApiId)
	}

	if len(s) > 0 {
		//有条件才截取最前面的and
		w = string([]byte(w)[4:])
		w = " where" + w
		return w, s
	} else {
		return "", nil
	}

}

func H5LogQueryAccessLogCount(ctx context.Context, req *apis.H5LogQueryAccessLogCountReq, reply *apis.H5LogQueryAccessLogCountReply) error {
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	limit := strconv.FormatInt((req.PageNumber-1)*req.PageSize, 10) + "," + strconv.FormatInt(req.PageSize, 10)

	where, val := getAccesslogCountWhere(req)
	sqlStr := "select create_time as date,merchant_name,api_name,count(state) as total ,sum(state) as success ,sum(is_reuse) as reuse,avg(elapsed) as elapsed  from t_gd_h5_access_log %s group by create_time,merchant_id,api_id order by create_time desc limit %s"

	sqlStr = fmt.Sprintf(sqlStr, where, limit)

	var ids []int

	count, err := orm.NewOrm().Raw("select id from t_gd_h5_access_log "+where+" group by create_time,merchant_id,api_id", val).QueryRows(&ids)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	reply.Total = count
	_, err = orm.NewOrm().Raw(sqlStr, val).QueryRows(&reply.H5QueryAccessLogCounts)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	for index, _ := range reply.H5QueryAccessLogCounts {
		reply.H5QueryAccessLogCounts[index].Failed = reply.H5QueryAccessLogCounts[index].Total - reply.H5QueryAccessLogCounts[index].Success
		reply.H5QueryAccessLogCounts[index].SuccessRate = strconv.FormatFloat(float64(100*reply.H5QueryAccessLogCounts[index].Success)/float64(reply.H5QueryAccessLogCounts[index].Total), 'f', 2, 64) + "%"
		reply.H5QueryAccessLogCounts[index].ReuseRate = strconv.FormatFloat(float64(100*reply.H5QueryAccessLogCounts[index].Reuse)/float64(reply.H5QueryAccessLogCounts[index].Total), 'f', 2, 64) + "%"
	}

	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	return nil
}
