package h5_access_log

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/errors"
	"github.com/astaxie/beego/orm"
	"time"
)

func getThirdpartAccesslogWhere(req *apis.H5LogQueryThirdpartAccessLogReq, querySet orm.QuerySeter) orm.QuerySeter {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		cond := orm.NewCondition()
		cond = cond.And("timestamp__gte", req.StartTimestamp).And("timestamp__lte", req.EndTimestamp)
		querySet = querySet.SetCond(cond)
	}
	if req.ProviderId != 0 {
		querySet = querySet.Filter("provider_id", req.ProviderId)
	}
	if req.PlateNo != "" {
		querySet = querySet.Filter("plate_no", req.PlateNo)
	}
	if req.Vin != "" {
		querySet = querySet.Filter("vin", req.Vin)
	}
	if req.ProviderApiId != 0 {
		querySet = querySet.Filter("provider_api_id", req.ProviderApiId)
	}

	if req.H5ApiId != 0 {
		querySet = querySet.Filter("h5_api_id", req.H5ApiId)
	}

	if req.H5ServiceId != 0 {
		querySet = querySet.Filter("h5_service_id", req.H5ServiceId)
	}

	return querySet
}

func H5LogQueryThirdpartAccessLog(ctx context.Context, req *apis.H5LogQueryThirdpartAccessLogReq, reply *apis.H5LogQueryThirdpartAccessLogReply) error {
	querySet := orm.NewOrm().QueryTable("t_gd_h5_thirdpart_access_log")
	querySet = getThirdpartAccesslogWhere(req, querySet)

	count, err := querySet.Count()
	if err != nil {
		return errors.DataBaseError
	}

	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	reply.Total = count
	_, err = querySet.OrderBy("-timestamp").Limit(req.PageSize, (req.PageNumber-1)*req.PageSize).All(&reply.H5ThirdpartLogs)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	for i, _ := range reply.H5ThirdpartLogs {
		var timeLayout = "2006-01-02 15:04:05"
		reply.H5ThirdpartLogs[i].CreateTime = time.Unix(reply.H5ThirdpartLogs[i].Timestamp, 0).Format(timeLayout)
	}

	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	return nil
	return nil
}
