package h5_access_log

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/errors"
	"fmt"
	"github.com/astaxie/beego/orm"
	"strconv"
)

func getThirdpartAccesslogCountWhere(req *apis.H5LogQueryThirdpartAccessLogCountReq) (w string, s []interface{}) {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		w += " and " + "timestamp between" + " ? and ?"
		s = append(s, req.StartTimestamp, req.EndTimestamp)
	}
	if req.ProviderApiId != 0 {
		w += " and " + "provider_api_id" + " = ?"
		s = append(s, req.ProviderApiId)
	}
	if req.ProviderId != 0 {
		w += " and " + "provider_id" + " = ?"
		s = append(s, req.ProviderId)
	}

	if req.H5ServiceId != 0 {
		w += " and " + "h5_service_id" + " = ?"
		s = append(s, req.H5ServiceId)
	}

	if req.H5ApiId != 0 {
		w += " and " + "h5_api_id" + " = ?"
		s = append(s, req.H5ApiId)
	}

	if len(s) > 0 {
		//有条件才截取最前面的and
		w = string([]byte(w)[4:])
		w = " where" + w
		return w, s
	} else {
		return "", nil
	}

}

func H5LogQueryThirdpartAccessLogCount(ctx context.Context, req *apis.H5LogQueryThirdpartAccessLogCountReq, reply *apis.H5LogQueryThirdpartAccessLogCountReply) error {
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	limit := strconv.FormatInt((req.PageNumber-1)*req.PageSize, 10) + "," + strconv.FormatInt(req.PageSize, 10)

	where, val := getThirdpartAccesslogCountWhere(req)
	sqlStr := "select create_time as date,provider_name,provider_api_name,count(state) as total,sum(state) as success,avg(elapsed) as elapsed  from t_gd_h5_thirdpart_access_log %s group by create_time,provider_id,provider_api_id order by create_time desc limit %s"

	sqlStr = fmt.Sprintf(sqlStr, where, limit)

	var ids []int

	count, err := orm.NewOrm().Raw("select id from t_gd_h5_thirdpart_access_log "+where+" group by create_time,provider_id,provider_api_id", val).QueryRows(&ids)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	reply.Total = count
	_, err = orm.NewOrm().Raw(sqlStr, val).QueryRows(&reply.H5QueryThirdpartAccessLogCounts)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	for index, _ := range reply.H5QueryThirdpartAccessLogCounts {
		reply.H5QueryThirdpartAccessLogCounts[index].Failed = reply.H5QueryThirdpartAccessLogCounts[index].Total - reply.H5QueryThirdpartAccessLogCounts[index].Success
		reply.H5QueryThirdpartAccessLogCounts[index].SuccessRate = strconv.FormatFloat(float64(100*reply.H5QueryThirdpartAccessLogCounts[index].Success)/float64(reply.H5QueryThirdpartAccessLogCounts[index].Total), 'f', 2, 64) + "%"
	}

	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize

	return nil
}
