package warning

import (
	"context"
	"gd_access_log/apis"
	"gd_access_log/rpc_apis"
	"gd_access_log/rpc_apis/gd_crontab"
)


// AccessLogCheck contains targets and api information for warning
type AccessLogCheck struct {
	ApiId        int64   `json:"api_id" bson:"api_id"`
	ApiName      string  `json:"api_name" bson:"api_name"`
	ApiRouter    string  `json:"api_router" bson:"api_router"`
	Count        int     `json:"count" bson:"count"`
	Success      int     `json:"success" bson:"success"`
	Valid        int     `json:"valid" bson:"valid"`
	NoRecord     int     `json:"no_record" bson:"no_record"`
	AvgElapsed   float64 `json:"avg_elapsed" bson:"avg_elapsed"`
	Elapsed      float64 `json:"avg_elapsed" bson:"elapsed"`
	FailRate     float64 `json:"fail_rate" bson:"fail_rate"`
	NorecordRate float64 `json:"norecord_rate" bson:"norecord_rate"`
}

// ThirdLogCheck contains targets and provider api information for warning
type ThirdLogCheck struct {
	ProviderName    string  `json:"provider_name" bson:"provider_name"`
	ProviderApiName string  `json:"provider_api_name" bson:"api_name"`
	ApiRouter       string  `json:"api_router" bson:"api_router"`
	Count           int     `json:"count" bson:"count"`
	Success         int     `json:"success" bson:"success"`
	Valid           int     `json:"valid" bson:"valid"`
	NoRecord        int     `json:"no_record" bson:"no_record"`
	AvgElapsed      float64 `json:"avg_elapsed" bson:"avg_elapsed"`
	ProviderApiId   int64   `json:"provider_api_id" bson:"provider_api_id"`
	Elapsed         float64 `json:"avg_elapsed" bson:"elapsed"`
	FailRate        float64 `json:"fail_rate" bson:"fail_rate"`
	NorecordRate    float64 `json:"norecord_rate" bson:"norecord_rate"`
}

type MgoAccessLog struct {
	ApiName   string  `json:"api_name" bson:"api_name"`
	Elapsed   float64 `json:"elapsed" bson:"elapsed"`
	State     int     `json:"state" bson:"state"`
	Code      int     `json:"code" bson:"code"`
	ApiId     int64   `json:"api_id" bson:"api_id"`
	Timestamp int64   `json:"timestamp" bson:"timestamp"`
}

type MgoThirdLog struct {
	ProviderApiName string  `json:"provider_api_name" bson:"provider_api_name"`
	ProviderName    string  `json:"provider_name" bson:"provider_name"`
	Elapsed         float64 `json:"elapsed" bson:"elapsed"`
	State           int     `json:"state" bson:"state"`
	Code            int     `json:"code" bson:"code"`
	ProviderApiId   int64   `json:"provider_api_id" bson:"provider_api_id"`
	Timestamp       int64   `json:"timestamp" bson:"timestamp"`
}

func AccessLogWarn(access *apis.AccessLog) {
	mReq := gd_crontab.ReciveAccessLogReq{}
	mReq.ApiName = access.ApiName
	if access.State == true {
		mReq.State = 1
	}
	mReq.Code = access.RawCode
	mReq.Elapsed = access.Elapsed
	mReq.ApiId = access.ApiId
	mReq.Timestamp = access.Timestamp
	_, err := rpc_apis.Crontab.ReciveAccessLog(context.Background(), &mReq)
	if err != nil {
		// TODO log
	}
}

func ThirdLogWarn(third *apis.ThirdpartLog) {
	mReq := gd_crontab.ReciveThirdPartLogReq{}
	mReq.ProviderApiName = third.ProviderApiName
	mReq.ProviderName = third.ProviderName
	if third.State == true {
		mReq.State = 1
	}
	mReq.Code = third.Code
	mReq.Elapsed = third.Elapsed
	mReq.ProviderApiId = third.ProviderApiId
	mReq.Timestamp = third.Timestamp
	_, err := rpc_apis.Crontab.ReciveThirdPartLog(context.Background(), &mReq)
	if err != nil {
		// TODO log
	}
}

/*
func LoopCheck() {
	go func() {
		t := time.NewTicker(time.Duration(LogTimeInterval) * time.Minute)
		for {
			select {
			case <-t.C:
				mutex.RLock()
				checkLog()
				cleanLog()
				mutex.RUnlock()
			}
		}
	}()
}*/
